@@ -1,7 +1,7 @@
 package Brackup;
 use strict;
 use vars qw($VERSION);
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 use Brackup::Config;
 use Brackup::ConfigSection;
@@ -15,3 +15,18 @@ use Brackup::Target;
 use Brackup::BackupStats;
 
 1;
+
+__END__
+
+=head1 NAME
+
+Brackup - Flexible backup tool.  Slices, dices, encrypts, and sprays across the net.
+
+=head1 FURTHER READING
+
+L<Brackup::Manual::Overview>
+
+L<brackup>
+
+L<brackup-restore>
+
