@@ -3,6 +3,8 @@ use strict;
 use warnings;
 use base 'Brackup::Target';
 use File::Path;
+use File::stat ();
+
 
 sub new {
     my ($class, $confsec) = @_;
@@ -49,11 +51,6 @@ sub chunkpath {
     return $self->_diskpath($dig, "chunk");
 }
 
-sub metapath {
-    my ($self, $dig) = @_;
-    return $self->_diskpath($dig, "meta");
-}
-
 sub has_chunk_of_handle {
     my ($self, $handle) = @_;
     my $dig = $handle->digest;  # "sha1:sdfsdf" format scalar
@@ -98,9 +95,13 @@ sub store_chunk {
     return 1;
 }
 
+sub _metafile_dir {
+    return $_[0]->{path}."/backups/";
+}
+
 sub store_backup_meta {
     my ($self, $name, $file) = @_;
-    my $dir = "$self->{path}/backups/";
+    my $dir = $self->_metafile_dir;
     unless (-d $dir) {
         mkdir $dir or die "Failed to mkdir $dir: $!\n";
     }
@@ -110,6 +111,46 @@ sub store_backup_meta {
     return 1;
 }
 
+sub backups {
+    my ($self) = @_;
+
+    my $dir = $self->_metafile_dir;
+    return () unless -d $dir;
+
+    opendir(my $dh, $dir) or
+        die "Failed to open $dir: $!\n";
+
+    my @ret = ();
+    while (my $fn = readdir($dh)) {
+        next unless $fn =~ s/\.brackup$//;
+        my $stat = File::stat::stat("$dir/$fn.brackup");
+        push @ret, Brackup::TargetBackupStatInfo->new($self, $fn,
+                                                      time => $stat->mtime,
+                                                      size => $stat->size);
+    }
+    closedir($dh);
+    return @ret;
+}
+
+# downloads the given backup name to the current directory (with
+# *.brackup extension)
+sub get_backup {
+    my ($self, $name) = @_;
+    my $dir  = $self->_metafile_dir;
+    my $file = "$dir/$name.brackup";
+    die "File doesn't exist: $file" unless -e $file;
+    open(my $in,  $file)            or die "Failed to open $file: $!\n";
+    open(my $out, ">$name.brackup") or die "Failed to open $name.brackup: $!\n";
+    my $buf;
+    my $rv;
+    while ($rv = sysread($in, $buf, 128*1024)) {
+        my $outv = syswrite($out, $buf);
+        die "copy error" unless $outv == $rv;
+    }
+    die "copy error" unless defined $rv;
+    return 1;
+}
+
 1;
 
 
